/*
 * Main.h
 *
 * Created: 27.12.2012 13:44:27
 *  Author: Nuckey
 */ 


#ifndef MAIN_PARTS_H_
#define MAIN_PARTS_H_
#include "Tester.PreConfig.h"
#include <avr/io.h>

//definitions of parts
#define PART_NONE 0
#define PART_DIODE 1
#define PART_TRANSISTOR 2
#define PART_FET 3
#define PART_TRIAC 4
#define PART_THYRISTOR 5
#define PART_RESISTOR 6
#define PART_CAPACITOR 7
#define PART_CELL 8

//End (parts)
//special definition for different parts
//FETs
#define PART_MODE_N_E_MOS 2
#define PART_MODE_P_E_MOS 3
#define PART_MODE_N_D_MOS 4
#define PART_MODE_P_D_MOS 5
#define PART_MODE_N_JFET 6
#define PART_MODE_P_JFET 7

//Bipolar
#define PART_MODE_NPN 1
#define PART_MODE_PNP 2


struct Diode_t
{
	uint8_t Anode;
	uint8_t Cathode;
	unsigned int Voltage;
}diodes[6];
extern struct Diode_t diodes[];


struct Trans_t
{
	unsigned long hfe[2];		//current amplification factor
	unsigned int uBE[2];		//B-E-voltage of the Transistor
	uint8_t b,c,e;		//pins of the Transistor
}trans;
extern struct Trans_t trans;

struct Cap_t
{
	unsigned long cval;		// capacitor value
	unsigned long cval_max;	//capacitor with maximum value
	unsigned long cval_uncorrected;	// capacity value without corrections
	#if FLASHEND > 0x1fff
	unsigned int esr;		// serial resistance of C in 0.01 Ohm
	#endif
	uint8_t ca, cb;		//pins of capacitor
	int8_t cpre;			//Prefix for capacitor value  -12=p, -9=n, -6=, -3=m
	int8_t cpre_max;		//Prefix of the biggest capacitor
}cap;
extern struct Cap_t cap;

struct Resis_t
{
	unsigned long rx;		// value of resistor RX
	#if FLASHEND > 0x1fff
	unsigned long lx;		// inductance uH
	#endif
	uint8_t ra,rb;		// Pins of RX
	uint8_t rt;			// Tristate-Pin (inactive)
}resis[3];
extern struct Resis_t resis[];


extern uint8_t NumOfDiodes;
extern unsigned int gthvoltage;		//Gate-threshold voltage

extern uint8_t PartReady;			//part detection is finished
extern uint8_t PartMode;
extern uint8_t tmpval, tmpval2;
extern unsigned int ref_mv;         //Reference-voltage  in mV units
extern uint8_t ResistorsFound;		//Number of found resistors
extern uint8_t ii;			// multipurpose counter

extern int16_t load_diff;		// difference voltage of loaded capacitor and internal reference

extern uint8_t WithReference;		// Marker for found precision voltage reference = 1
extern uint8_t PartFound;	 	// the found part
extern char outval[12];		// String for ASCII-outpu
extern uint8_t empty_count;		// counter for max count of empty measurements
extern uint8_t mess_count;		// counter for max count of nonempty measurements

#if POWER_OFF+0 > 1
extern unsigned int display_time;	// display time of measurement in ms units
#endif

#ifdef AUTO_CAL
extern   uint16_t resis680pl;
extern   uint16_t resis680mi;
extern   uint8_t pin_combination;		// coded Pin-combination  2:1,3:1,1:2,x:x,3:2,1:3,2:3
#endif



#endif /* MAIN_PARTS_H_ */