/*
 * Main.c
 *
 * Created: 26.12.2012 09:32:35
 *  Author: Nuckey
 */ 


#include <avr/io.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <stdlib.h>
#include <string.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <math.h>



#include "Main.Methoden.h"
#include "Tester.PreConfig.h"
#include "Lcd.h"
#include "stdlib.h"
#include "Tester.Language.h"
#include "CheckPins.h"
#include "AutoCheck.h"


//******************************************************************
// output of flux voltage for 1-2 diodes in row 2
// bcdnum = Numbers of both Diodes:
// higher 4 Bit  number of first Diode
// lower 4 Bit  number of second Diode (Structure diodes[nn])
// if number >= 3  no output is done
void UfAusgabe(uint8_t bcdnum) 
{
   lcd_line2(); 					//2. row
   //lcd_fix_string(Uf_str);			//"Uf="
   mVAusgabe(bcdnum >> 4);
   mVAusgabe(bcdnum & 0x0f);
}

void mVAusgabe(uint8_t nn) 
{
   if (nn < 3) {
      // Output in mV units
      //DisplayValue(diodes[nn].Voltage,-3,'V',3);
      lcd_space();
   }
}

//#			 output of resistor value		#
void RvalOut(uint8_t ii) 
{	
   //DisplayValue(resis[ii].rx,-1,LCD_CHAR_OMEGA,4);
   lcd_space();
 }
 
void ChargePin10ms(uint8_t PinToCharge, uint8_t ChargeDirection) 
{
	//Load the specified pin to the specified direction with 680 Ohm for 10ms.
	//Will be used by discharge of MOSFET Gates or to load big capacities.
	//Parameters:
	//PinToCharge: specifies the pin as mask for R-Port
	//ChargeDirection: 0 = switch to GND (N-Kanal-FET), 1= switch to VCC(P-Kanal-FET)

	if(ChargeDirection&1) {
		R_PORT |= PinToCharge;	//R_PORT to 1 (VCC)
	} else {
		R_PORT &= ~PinToCharge; // or 0 (GND)
	}
	R_DDR |= PinToCharge;		//switch Pin to output, across R to GND or VCC
	wait10ms();
	// switch back Input, no current
	R_DDR &= ~PinToCharge;	// switch back to input
	R_PORT &= ~PinToCharge;	// no Pull up
}


// first discharge any charge of capacitors
void EntladePins() 
{
	uint8_t adc_gnd;										// Mask of ADC-outputs, which can be directly connected to GND
	unsigned int adcmv[3];									// voltages of 3 Pins in mV
	unsigned int clr_cnt;									// Clear Counter
	uint8_t lop_cnt;										// loop counter
	
	for(lop_cnt=0;lop_cnt<10;lop_cnt++) 
	{
		adc_gnd = TXD_MSK;									// put all ADC to Input
		ADC_DDR = adc_gnd;
		ADC_PORT = TXD_VAL;									// ADC-outputs auf 0
		R_PORT = 0;											// R-outputs auf 0
		R_DDR = (2<<(PC2*2)) | (2<<(PC1*2)) | (2<<(PC0*2)); // R_H for all Pins to GND
		adcmv[0] = W5msReadADC(PC0);						// which voltage has Pin 1?
		adcmv[1] = ReadADC(PC1);							// which voltage has Pin 2?
		adcmv[2] = ReadADC(PC2);							// which voltage has Pin 3?
		
		if ((PartFound == PART_CELL) || (adcmv[0] < CAP_EMPTY_LEVEL) & (adcmv[1] < CAP_EMPTY_LEVEL) & (adcmv[2] < CAP_EMPTY_LEVEL)) 
		{
			ADC_DDR = TXD_MSK;								// switch all ADC-Pins to input
			R_DDR = 0;										// switch all R_L Ports (and R_H) to input
			return;
		}
		
		// all Pins with voltage lower than 1V can be connected directly to GND (ADC-Port)
		if (adcmv[0] < 1000) 
		{
			adc_gnd |= (1<<PC0);							//Pin 1 directly to GND
		}
		if (adcmv[1] < 1000) 
		{
			adc_gnd |= (1<<PC1);							//Pin 2 directly to GND
		}
		if (adcmv[2] < 1000) 
		{
			adc_gnd |= (1<<PC2);							//Pin 3 directly to  GND
		}
		ADC_DDR = adc_gnd;									// switch all selected ADC-Ports at the same time
		
		// additionally switch the leaving Ports with R_L to GND.
		// since there is no disadvantage for the already directly switched pins, we can
		// simply switch all  R_L resistors to GND
		R_DDR = (1<<(PC2*2)) | (1<<(PC1*2)) | (1<<(PC0*2));	// Pins across R_L resistors to GND
		
		for(clr_cnt=0;clr_cnt<MAX_ENTLADE_ZEIT;clr_cnt++) 
		{
			wdt_reset();
			adcmv[0] = W20msReadADC(PC0);					// which voltage has Pin 1?
			adcmv[1] = ReadADC(PC1);						// which voltage has Pin 2?
			adcmv[2] = ReadADC(PC2);						// which voltage has Pin 3?
			if (adcmv[0] < 1300) 
			{
				ADC_DDR |= (1<<PC0);						// below 1.3V , switch directly with ADC-Port to GND
			}
			if (adcmv[1] < 1300) 
			{
				ADC_DDR |= (1<<PC1);						// below 1.3V, switch directly with ADC-Port to GND
			}
			if (adcmv[2] < 1300) 
			{
				ADC_DDR |= (1<<PC2);						// below 1.3V, switch directly with ADC-Port to GND
			}
			if ((adcmv[0] < (CAP_EMPTY_LEVEL+2)) && (adcmv[1] < (CAP_EMPTY_LEVEL+2)) && (adcmv[2] < (CAP_EMPTY_LEVEL+2))) 
			{
				break;
			}
		}
		
		if (clr_cnt == MAX_ENTLADE_ZEIT) 
		{
			PartFound = PART_CELL;							// mark as Battery
															// there is charge on capacitor, warn later!
		}
		for(adcmv[0]=0;adcmv[0]<clr_cnt;adcmv[0]++) 
		{	
			wait1ms();										// for safety, discharge 5% of discharge  time
		}
	}
}

void RefVoltage(void) 
{
	// RefVoltage interpolates table RHtab corresponding to voltage ref_mv .
	// RHtab contain the factors to get capacity from load time with 470k for
	// different Band gab reference voltages.
	// for remember:
	//resistor     470000 Ohm      1000 1050 1100 1150 1200 1250 1300 1350 1400  mV
	//uint16_t RHTAB[] MEM_TEXT = { 954, 903, 856, 814, 775, 740, 707, 676, 648};



	unsigned int referenz;
	unsigned int y1, y2;
	uint8_t tabind;
	uint8_t tabres;

				#ifdef AUTO_CAL
referenz = ref_mv + (int16_t)eeprom_read_word((uint16_t *)(&ref_offset));
#else
referenz = ref_mv + REF_C_KORR;
	#endif

	if (referenz >= Ref_Tab_Beginn) 
	{
		referenz -= Ref_Tab_Beginn;
	} else  
	{
		referenz = 0;		// limit to begin of table
	}

	tabind = referenz / Ref_Tab_Abstand;
	tabres = referenz % Ref_Tab_Abstand;
	tabres = Ref_Tab_Abstand-tabres;
	if (tabind > 7) 
	{
		tabind = 7;		// limit to end of table
	}
	// interpolate the table of factors
	y1 = MEM_read_word(&RHtab[tabind]);
	y2 = MEM_read_word(&RHtab[tabind+1]);
	// RHmultip is the interpolated factor to compute capacity from load time with 470k
	RHmultip = ((y1 - y2) * tabres + (Ref_Tab_Abstand/2)) / Ref_Tab_Abstand + y2;
 }
 
void lcd_clear_line(void) 
 {
	 // writes 20 spaces to LCD-Display, Cursor must be positioned to first column
	 unsigned char ll;
	 for (ll=0;ll<20;ll++) 
	 {
		 lcd_space();
	 }
 }
 
 
//###############################################################################
//#					display of values and units									#
//#																				#
//#	- max. 4 digits excluding "." and unit										#
//#																				#
//#  requires:																	#
//#  - value																	#
//#  - exponent of factor related to base unit (value * 10^x)					#
//#   e.g: p = 10^-12 -> -12													#
//#  - unit character (0 = none)												#
//#  digits = 2, 3 or 4															#
//###############################################################################
void DisplayValue(unsigned long Value, int8_t Exponent, unsigned char Unit, unsigned char digits)
{
  char OutBuffer[15];
  unsigned int      Limit;
  unsigned char     Prefix;					/* prefix character */
  uint8_t           Offset;					/* exponent of offset to next 10^3 step */
  uint8_t           Index;					/* index ID */
  uint8_t           Length;					/* string length */


  Limit = 100;								/* scale value down to 2 digits */
  if (digits == 3) Limit = 1000;
  if (digits == 4) Limit = 10000;
  
  while (Value >= Limit)
  {
    Value += 5;								/* for automagic rounding */
    Value = Value / 10;						/* scale down by 10^1 */
    Exponent++;								/* increase exponent by 1 */
  }


  
  //#				determine prefix				#
  Length = Exponent + 12;
  if (Length <  0) Length = 0;				/* Limit to minimum prefix */
  if (Length > 18) Length = 18;				/* Limit to maximum prefix */
  Index = Length / 3;
  Offset = Length % 3;
    if (Offset > 0)
    {
      Index++;								/* adjust index for exponent offset, take next prefix */
      Offset = 3 - Offset;					/* reverse value (1 or 2) */
    }
  Prefix = MEM_read_byte((uint8_t *)(&PrefixTab[Index]));   /* look up prefix in table */
  
   //#				display value					#

  /* convert value into string */
  utoa((unsigned int)Value, OutBuffer, 10);
  Length = strlen(OutBuffer);

  /* position of dot */
  Exponent = Length - Offset;		/* calculate position */

  if (Exponent <= 0)			/* we have to prepend "0." */
  {
    /* 0: factor 10 / -1: factor 100 */
//    lcd_data('0');
    lcd_data('.');
    if (Exponent < 0) lcd_data('0');	/* extra 0 for factor 100 */
  }

  if (Offset == 0) Exponent = -1;	/* disable dot if not needed */

  /* adjust position to array or disable dot if set to 0 */
//  Exponent--;

  /* display value and add dot if requested */
  Index = 0;
  while (Index < Length)		/* loop through string */
  {
    lcd_data(OutBuffer[Index]);		/* display char */
    Index++;				/* next one */
    if (Index == Exponent) {
      lcd_data('.');			/* display dot */
    }
  }

  /* display prefix and unit */
  if (Prefix != 0) lcd_data(Prefix);
  if (Unit) lcd_data(Unit);
}